@@ -41,6 +41,8 @@
 #' @param keepNA If \code{TRUE}, NA values are converted to #N/A in Excel else NA cells will be empty.
 #' @seealso \code{\link{writeDataTable}}
 #' @export writeData
+#' @details Formulae written using writeFormula to a Workbook object will not get picked up by read.xlsx().
+#' This is because only the formula is written and left to Excel to evaluate the formula when the file is opened in Excel.
 #' @rdname writeData
 #' @examples
 #' 
@@ -83,7 +85,21 @@
 #' names(v) <- paste("Hyperlink", 1:4) # Names will be used as display text
 #' class(v) <- 'hyperlink'
 #' writeData(wb, "Cars", x = v, xy = c("B", 32))
-
+#'
+#'
+#'addWorksheet(wb, "Formula")
+#' 
+#' ## create column of class "formula"
+#' df <- data.frame(x=1:3, y = 1:3,
+#'                  z = paste(paste0("A", 1:3+1L), paste0("B", 1:3+1L), sep = " + "),
+#'                  stringsAsFactors = FALSE)
+#' 
+#' class(df$z) <- c(class(df$z), "formula")
+#' 
+#' writeData(wb, sheet = "Formula", x = df)
+#' 
+#' 
+#'
 #' ## Save workbook
 #' saveWorkbook(wb, "writeDataExample.xlsx", overwrite = TRUE)
 writeData <- function(wb, 
@@ -141,6 +157,13 @@ writeData <- function(wb,
     colNames = FALSE
   }
   
+  ## special case - formula
+  if("formula" %in% class(x)){
+    x <- data.frame("X" = x, stringsAsFactors = FALSE)
+    class(x[[1]]) <- "formula"
+    colNames = FALSE
+  }
+
   if(is.vector(x) | is.factor(x))
     colNames <- FALSE ## this will go to coerce.default and rowNames will be ignored 
   
@@ -157,8 +180,14 @@ writeData <- function(wb,
   colClasses <- lapply(x, function(x) tolower(class(x)))
   
 
-  ## write autoFilter, can only have a single filter per worksheet
+  
   sheetX <- wb$validateSheet(sheet)
+  if(wb$isChartSheet[[sheetX]]){
+    stop("Cannot write to chart sheet.")
+    return(NULL)
+  }
+    
+  ## write autoFilter, can only have a single filter per worksheet
   if(withFilter)
     wb$worksheets[[sheetX]]$autoFilter <- sprintf('<autoFilter ref="%s"/>', paste(getCellRefs(data.frame("x" = c(startRow, startRow), "y" = c(startCol, startCol + nCol - 1L))), collapse = ":"))
   
@@ -236,3 +265,98 @@ writeData <- function(wb,
   invisible(0)
   
 }
+
+
+
+
+
+
+
+
+
+
+
+
+#' @name writeFormula
+#' @title Write a character vector as an Excel Formula
+#' @author Alexander Walker
+#' @description Write a a character vector containing Excel formula to a worksheet
+#' @param wb A Workbook object containing a worksheet.
+#' @param sheet The worksheet to write to. Can be the worksheet index or name.
+#' @param x A character vector.
+#' @param startCol A vector specifiying the starting column to write to.
+#' @param startRow A vector specifiying the starting row to write to.
+#' @param xy An alternative to specifying \code{startCol} and
+#' \code{startRow} individually.  A vector of the form
+#' \code{c(startCol, startRow)}.
+#' @seealso \code{\link{writeData}}
+#' @export writeFormula
+#' @rdname writeFormula
+#' @examples
+#' 
+#' ## There are 3 ways to write a formula
+#' 
+#' wb <- createWorkbook()
+#' addWorksheet(wb, "Sheet 1")
+#' writeData(wb, "Sheet 1", x = iris)
+#' 
+#' ## SEE int2col() to convert int to Excel column label
+#' 
+#' ## 1. -  As a character vector using writeFormula
+#' 
+#' v <- c("SUM(A2:A151)", "AVERAGE(B2:B151)") ## skip header row
+#' writeFormula(wb, sheet = 1, x = v, startCol = 10, startRow = 2)
+#' writeFormula(wb, 1, x = "A2 + B2", startCol = 10, startRow = 10)
+#' 
+#' 
+#' ## 2. - As a data.frame column with class "formula" using writeData
+#' 
+#' df <- data.frame(x=1:3,
+#'                  y = 1:3,
+#'                  z = paste(paste0("A", 1:3+1L), paste0("B", 1:3+1L), sep = " + "),
+#'                  z2 = sprintf("ADDRESS(1,%s)", 1:3),
+#'                  stringsAsFactors = FALSE)
+#' 
+#' class(df$z) <- c(class(df$z), "formula")
+#' class(df$z2) <- c(class(df$z2), "formula")
+#' 
+#' addWorksheet(wb, "Sheet 2")
+#' writeData(wb, sheet = 2, x = df)
+#' 
+#' 
+#' 
+#' ## 3. - As a vector with class "formula" using writeData
+#' 
+#' v2 <- c("SUM(A2:A4)", "AVERAGE(B2:B4)", "MEDIAN(C2:C4)")
+#' class(v2) <- c(class(v2), "formula")
+#' 
+#' writeData(wb, sheet = 2, x = v2, startCol = 10, startRow = 2)
+#' 
+#' ## Save workbook
+#' saveWorkbook(wb, "writeFormulaExample.xlsx", overwrite = TRUE)
+#' 
+writeFormula <- function(wb, 
+                         sheet,
+                         x,
+                         startCol = 1,
+                         startRow = 1, 
+                         xy = NULL){
+
+  
+  dfx <- data.frame("X" = x, stringsAsFactors = FALSE)
+  class(dfx$X) <- c("character", "formula")
+  
+  writeData(wb = wb,
+            sheet = sheet,
+            x = dfx,
+            startCol = startCol,
+            startRow = startRow, 
+            xy = xy,
+            colNames = FALSE,
+            rowNames = FALSE)
+  
+  
+  invisible(0)
+  
+}
+
